/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bpel.engine.env;

import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;

import org.jdom.JDOMException;
import org.jdom.output.DOMOutputter;
import org.objectweb.fractal.fraclet.extensions.Membrane;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import org.ow2.petals.bpel.engine.message.JBIContextImpl;
import org.ow2.petals.bpel.engine.message.JBIExternalMessageImpl;
import org.ow2.petals.bpel.engine.util.Sender.SendStyle;
import org.ow2.petals.commons.threadlocal.DocumentBuilders;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.UtilFactory;

import com.ebmwebsourcing.easybpel.model.bpel.api.BPELException;
import com.ebmwebsourcing.easybpel.model.bpel.impl.exception.UserDefinedException;
import com.ebmwebsourcing.easybpel.model.bpel.impl.message.BPELInternalMessageImpl;
import com.ebmwebsourcing.easyviper.core.api.CoreException;
import com.ebmwebsourcing.easyviper.core.api.env.ExternalContext;
import com.ebmwebsourcing.easyviper.core.api.env.Sender;
import com.ebmwebsourcing.easyviper.core.api.soa.Endpoint;
import com.ebmwebsourcing.easyviper.core.api.soa.message.InternalMessage;
import com.ebmwebsourcing.easyviper.core.impl.env.AbstractSenderImpl;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
@org.objectweb.fractal.fraclet.annotations.Component
@Membrane(controller = "primitive")
public class JBISenderImpl extends AbstractSenderImpl implements Sender {

    /**
     *
     */
    private static final long serialVersionUID = 1L;

    @SuppressWarnings("unchecked")
    public void sendTo(final InternalMessage<?> internalMessage, final Endpoint providerEndpoint,
            final ExternalContext context) throws CoreException {
        final JBIContextImpl jbiContext = (JBIContextImpl) context;
        try {
            // retrieve the jbi context

            if (providerEndpoint.getInvokedOperation() == null) {
                providerEndpoint.setInvokedOperation(jbiContext.getInitialExchange().getOperation()
                        .getLocalPart());
            }

            // format message
            final InternalMessage formattedInternalMessage = (BPELInternalMessageImpl) internalMessage;

            JBIExternalMessageImpl externalMessage = (JBIExternalMessageImpl) this.messageConverter
                    .createExternalMessageFromInternalMessage(formattedInternalMessage);

            jbiContext.getListener().getLogger().finest(
                    "Asynchronous send: PROVIDER ENDPOINT NAME: "
                            + providerEndpoint.getEndpointName());
            if (externalMessage != null) {
                jbiContext.getListener().getLogger().finest(
                        "MESSAGE SENT BY BPEL ENGINE: \n" + externalMessage);
            } else {
                final Document doc = DocumentBuilders.getNamespaceDocumentBuilder().newDocument();
                final Element elmt = doc.createElement("null");
                doc.appendChild(elmt);
                externalMessage = new JBIExternalMessageImpl();
                externalMessage.setContent(doc.getDocumentElement());
                jbiContext.getListener().getLogger().finest(
                        "MESSAGE SENT BY BPEL ENGINE: \n" + externalMessage);
            }

            if (!providerEndpoint.getEndpointName().equals("client")) {

                // retrieve the sender
                final org.ow2.petals.bpel.engine.util.Sender sender = jbiContext.getSender();

                // send the message
                sender.send(externalMessage, new Long(0), providerEndpoint, context,
                        SendStyle.ASYNCHRONOUS_SEND);

            } else {
                // set the response in exchange

                // retrieve the exchange
                final Exchange exchange = jbiContext.getInitialExchange();

                // set the response
                exchange.getOutMessage().setContent(
                        UtilFactory.getSourceUtil().createDOMSource(
                                externalMessage.getContent().getOwnerDocument()));

                jbiContext.getListener().send(exchange);

                // // notify the listener
                // jbiContext.setFinished(true);
            }
        } catch (final MessagingException e) {
            jbiContext.getListener().getLogger().warning("Error on invoke: " + e.getMessage());
            throw new CoreException(e);
        } catch (final BPELException e) {
            jbiContext.getListener().getLogger().warning("Error on invoke: " + e.getMessage());
            throw new CoreException(e);
        }
    }

    @SuppressWarnings("unchecked")
    public InternalMessage<?> sendSyncTo(final InternalMessage<?> message,
            final Endpoint providerEndpoint, final ExternalContext context) throws CoreException {
        InternalMessage<?> internalMessage = null;
        // retrieve the jbi context
        final JBIContextImpl jbiContext = (JBIContextImpl) context;
        try {

            if (providerEndpoint.getInvokedOperation() == null) {
                providerEndpoint.setInvokedOperation(jbiContext.getInitialExchange().getOperation()
                        .getLocalPart());
            }

            // format message
            final InternalMessage formattedInternalMessage = (BPELInternalMessageImpl) message;

            jbiContext.getListener().getLogger().finest(
                    "Synchronous send: PROVIDER ENDPOINT NAME: "
                            + providerEndpoint.getEndpointName());
            JBIExternalMessageImpl externalMessage = (JBIExternalMessageImpl) this.messageConverter
                    .createExternalMessageFromInternalMessage(formattedInternalMessage);
            if (externalMessage != null) {
                jbiContext.getListener().getLogger().finest(
                        "MESSAGE SENT SYNC BY BPEL engine: \n" + externalMessage);
            } else {
                final Document doc = DocumentBuilders.getNamespaceDocumentBuilder().newDocument();
                final Element elmt = doc.createElement("null");
                doc.appendChild(elmt);
                externalMessage = new JBIExternalMessageImpl();
                externalMessage.setContent(doc.getDocumentElement());
                jbiContext.getListener().getLogger().finest(
                        "MESSAGE SENT SYNC BY BPËL engine: \n" + externalMessage);
            }

            // retrieve the sender
            final org.ow2.petals.bpel.engine.util.Sender sender = jbiContext.getSender();

            // send the message
            final JBIExternalMessageImpl responseMessage = sender.send(externalMessage,
                    new Long(0), providerEndpoint, context, SendStyle.SYNCHRONOUS_SEND);

            internalMessage = this.messageConverter
                    .createInternalMessageFromExternalMessage(responseMessage);
            if (internalMessage != null) {
                jbiContext.getListener().getLogger().finest(
                        "MESSAGE RECEIVED BY BPEL engine: \n" + internalMessage);
            } else {
                jbiContext.getListener().getLogger().finest(
                        "MESSAGE RECEIVED BY BPEL engine: \n" + internalMessage);
            }
        } catch (final BPELException e) {
            jbiContext.getListener().getLogger().warning("Error on invoke: " + e.getMessage());
            throw new CoreException(e);
        }
        return internalMessage;
    }

    public void sendTo(CoreException e, ExternalContext context) throws CoreException {
        try {
            // retrieve the jbi context
            final JBIContextImpl jbiContext = (JBIContextImpl) context;

            // retrieve the exchange
            final Exchange exchange = jbiContext.getInitialExchange();

            // set the response
            jbiContext.getListener().getLogger().warning("Send error: " + exchange.getPattern());
            if ((MEPConstants.IN_OUT_PATTERN.equals(exchange.getPattern()))
                    || (MEPConstants.IN_OPTIONAL_OUT_PATTERN.equals(exchange.getPattern()))) {
                if (e instanceof UserDefinedException) {
                    org.jdom.Element internalMsg = (org.jdom.Element) ((UserDefinedException) e)
                            .getFault().getContent();

                    // convert jdom to dom
                    DOMOutputter converter = new DOMOutputter();

                    org.w3c.dom.Document domDocument = converter.output(internalMsg.getDocument());

                    Fault fault = exchange.createFault();
                    fault.setContent(UtilFactory.getSourceUtil().createDOMSource(domDocument));
                    exchange.setFault(fault);
                    jbiContext.getListener().getLogger().finest("Send business fault: \n" + e);
                } else {
                    jbiContext.getListener().getLogger().finest("Send technical fault: \n" + e);
                    MessagingException me = new MessagingException(e.getMessage());
                    me.setStackTrace(e.getStackTrace());
                    exchange.setFault(me);
                }

                // notify the listener
                jbiContext.getListener().send(exchange);
            }
        } catch (JDOMException e1) {
            throw new CoreException(e1);
        } catch (MessagingException e1) {
            throw new CoreException(e1);
        }
    }
}
